from os.path import join, expanduser, curdir, abspath
import unittest
import agfusion
from agfusion import utils
import pyensembl
from Bio import SeqIO

data = pyensembl.EnsemblRelease(84,'mouse')
db = agfusion.AGFusionDB(abspath(join(curdir,'agfusion.mus_musculus.84.db')))
db.build = 'mus_musculus_84'

class TestSequencePrediction(unittest.TestCase):
    def test_1(self):
        """
        test CDS and cDNA correct for junction that is on exon boundaries and
        produces an in-frame protein.
        """

        #test the dna and protein coding sequences are correct by comparing
        #with manually generally sequences

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000022770",
            gene5primejunction=31684294,
            gene3prime="ENSMUSG00000002413",
            gene3primejunction=39648486,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )

        fusion.save_transcript_cdna('DLG1-BRAF_mouse')
        fusion.save_transcript_cds('DLG1-BRAF_mouse')
        fusion.save_proteins('DLG1-BRAF_mouse')
        #fusion.save_images('DLG1-BRAF_mouse')

        test_cdna = SeqIO.parse(open('./data/Dlg1-Braf_cdna_manual.fa','r'),'fasta')
        test_cds = SeqIO.parse(open('./data/Dlg1-Braf_cds_manual.fa','r'),'fasta')

        expected_transcript_combinations = [
            'ENSMUST00000100001-ENSMUST00000002487',
            'ENSMUST00000064477-ENSMUST00000002487',
            'ENSMUST00000115205-ENSMUST00000002487',
            'ENSMUST00000023454-ENSMUST00000002487',
            'ENSMUST00000115201-ENSMUST00000002487',
            'ENSMUST00000132176-ENSMUST00000002487'
        ]

        assert len(set(fusion.transcripts.keys()).intersection(set(expected_transcript_combinations)))==6, "Test 1: unexpected number protein coding transcripts."

        for seq in test_cdna:
            trans=fusion.transcripts[str(seq.id)]
            assert seq.seq==trans.cdna.seq, "cDNA is wrongly predicted: %s" % seq.id

        for seq in test_cds:
            trans=fusion.transcripts[str(seq.id)]
            assert seq.seq==trans.cds.seq, "cds is wrongly predicted: %s" % seq.id

    def test_2(self):
        """
        Test CDS correct for junction within the exon (not on boundary) for two
        genes on reverse strand
        """

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000002413",
            gene5primejunction=39725110,
            gene3prime="ENSMUSG00000002413",
            gene3primejunction=39610402,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )

        cds = 'ATGGCGGCGCTGAGTGGCGGCGGTGGCAGCAGCAGCGGTGGCGGCGGCGGCGGTGGCGGCGGCGG' + \
              'TGGCGGTGGCGACGGCGGCGGCGGCGCCGAGCAGGGCCAGGCTCTGTTCAATGGCGACATGGAGC' + \
              'CGGAGGCCGGCGCTGGCGCCGCGGCCTCTTCGGCTGCGGACCCGGCCATTCCTGAAGAATTTGCAGCCTTCAAGTAG'

        assert str(fusion.transcripts['ENSMUST00000002487-ENSMUST00000002487'].cds.seq)==cds, "Test 2: CDS wrong"

    def test_3(self):
        """
        Test CDS correct for junction within the exon (not on boundary) for one gene
        one the forward and one gene on the reverse strand
        """

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000022770",
            gene5primejunction=31664869,
            gene3prime="ENSMUSG00000002413",
            gene3primejunction=39610402,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )

        cds = 'ATGCCGGTCCGGAAGCAAGAATTTGCAGCCTTCAAGTAG'

        assert str(fusion.transcripts['ENSMUST00000064477-ENSMUST00000002487'].cds.seq)==cds, "Test 3: CDS wrong"

    def test_mouse_4(self):
        """
        Test cDNA correctly produced for junctions being in UTRs
        """

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000022770",
            gene5primejunction=31664851,
            gene3prime="ENSMUSG00000022770",
            gene3primejunction=31873343,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )

        cdna = 'GGGGGTGCGGCCGCCGAAGGGGGAGCTCCTCCCCCGTCCCCTCACCCCCTCAGCTGAGCT' + \
              'CGGGGCGGGGCGGGGTACGTGGAGCGGGGCCGGGCGGGGAAGCTGCTCCGAGTCCGGCCG' + \
              'GAGCGCACCCGGGGCGCCCGCGTACGCCGCTCGCGGGAACTTTGCGGCGGAGCCGCAGGT' + \
              'GTGGAGGCCGCGGAGGGGGGTGCATGAGCGGCGCGGAGAGCGGCGGCTGTCCGGTCCGGC' + \
              'CCCTGCTGGAGTCGCCGCCGGGAGGAGACGAACGAGGAACCAG' + \
              'GTGTGTGCCGCCTTCCTGATTCTGGAGAAAA' + \
              'AAAA'

        assert str(fusion.transcripts['ENSMUST00000064477-ENSMUST00000064477'].cdna.seq)==cdna, "Test 4: cDNA wrong"

    def test_mouse_5(self):

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000022770",
            gene5primejunction=31664850,
            gene3prime="ENSMUSG00000002413",
            gene3primejunction=39603240,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )

        cdna = 'GGGGGTGCGGCCGCCGAAGGGGGAGCTCCTCCCCCGTCCCCTCACCCCCTCAGCTGAGCT' + \
              'CGGGGCGGGGCGGGGTACGTGGAGCGGGGCCGGGCGGGGAAGCTGCTCCGAGTCCGGCCG' + \
              'GAGCGCACCCGGGGCGCCCGCGTACGCCGCTCGCGGGAACTTTGCGGCGGAGCCGCAGGT' + \
              'GTGGAGGCCGCGGAGGGGGGTGCATGAGCGGCGCGGAGAGCGGCGGCTGTCCGGTCCGGC' + \
              'CCCTGCTGGAGTCGCCGCCGGGAGGAGACGAACGAGGAACCAG' + \
              'GTGTGTGCCGCCTTCCTGATTCTGGAGAAA' + \
              'GAAA'

        assert str(fusion.transcripts['ENSMUST00000064477-ENSMUST00000002487'].cdna.seq)==cdna,"Test 10: incorrect cDNA"

class TestEffectPrediciton(unittest.TestCase):
    def test_mouse_1(self):
        """
        Test that AGFusion determines if the effect on each individual transcript
        """

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000022770",
            gene5primejunction=31664852,
            gene3prime="ENSMUSG00000002413",
            gene3primejunction=39651764,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )

        assert fusion.transcripts['ENSMUST00000064477-ENSMUST00000002487'].effect_5prime=='CDS (start)',"Test 5: not CDS start"
        assert fusion.transcripts['ENSMUST00000064477-ENSMUST00000002487'].effect_3prime=='CDS',"Test 5: not CDS"

    def test_mouse_2(self):

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000022770",
            gene5primejunction=31664851,
            gene3prime="ENSMUSG00000002413",
            gene3primejunction=39610381,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )

        assert fusion.transcripts['ENSMUST00000064477-ENSMUST00000002487'].effect_5prime=='5UTR (end)',"Test 6: Not found in 5'UTR-end"
        assert fusion.transcripts['ENSMUST00000064477-ENSMUST00000002487'].effect_3prime=='3UTR (start)', "Test 6: Not found in at 3'UTR beginning"

    def test_mouse_3(self):

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000022770",
            gene5primejunction=31664850,
            gene3prime="ENSMUSG00000002413",
            gene3primejunction=39610381,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )
        assert fusion.transcripts['ENSMUST00000064477-ENSMUST00000002487'].effect_5prime=='5UTR',"Test 7: Not found in 5'UTR"
        assert fusion.transcripts['ENSMUST00000064477-ENSMUST00000002487'].effect_3prime=='3UTR (start)', "Test 7: Not found in at 3'UTR beginning"

    def test_mouse_4(self):

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000022770",
            gene5primejunction=31664851,
            gene3prime="ENSMUSG00000022770",
            gene3primejunction=31871782,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )

        assert fusion.transcripts['ENSMUST00000064477-ENSMUST00000064477'].effect_5prime=='5UTR (end)',"Test 8: Not found in 5'UTR-end"
        assert fusion.transcripts['ENSMUST00000064477-ENSMUST00000064477'].effect_3prime=='3UTR (start)', "Test 8: Not found in at 3'UTR beginning"

    def test_mouse_5(self):

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000002413",
            gene5primejunction=39725296,
            gene3prime="ENSMUSG00000002413",
            gene3primejunction=39610381,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )

        assert fusion.transcripts['ENSMUST00000002487-ENSMUST00000002487'].effect_5prime=='5UTR (end)',"Test 9: Not found in 5'UTR-end"
        assert fusion.transcripts['ENSMUST00000002487-ENSMUST00000002487'].effect_3prime=='3UTR (start)', "Test 9: Not found in at 3'UTR beginning"

    def test_mouse_6(self):

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000022770",
            gene5primejunction=31743271,
            gene3prime="ENSMUSG00000002413",
            gene3primejunction=39665003,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )
        t = fusion.transcripts['ENSMUST00000023454-ENSMUST00000002487']

        assert t.effect_5prime=="intron (cds)","Test 11: incorrect 5' effect: %s" % t.effect_5prime
        assert t.effect_3prime=="intron (cds)","Test 11: incorrect 3' effect: %s" % t.effect_3prime

    def test_mouse_7(self):

        fusion = agfusion.Fusion(
            gene5prime="ENSMUSG00000022770",
            gene5primejunction=31664820,
            gene3prime="ENSMUSG00000002413",
            gene3primejunction=39610405,
            db=db,
            pyensembl_data=data,
            protein_databases=['pfam', 'tmhmm'],
            noncanonical=True
        )
        t = fusion.transcripts['ENSMUST00000023454-ENSMUST00000002487']

        assert t.effect_5prime=="intron (before cds)","Test 12: incorrect 5' effect: %s" % t.effect_5prime
        assert t.effect_3prime=="intron (cds)","Test 12: incorrect 3' effect: %s" % t.effect_3prime

if __name__ == "__main__":
    unittest.main()



import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import matplotlib.pyplot
from itertools import cycle

# this is so I can plot graphics on a headless server

matplotlib.pyplot.ioff()

HORIZONTAL_LEVELS = [1,2,3,4]


class _Plot(object):
    def __init__(self, filename='', height=0, width=0, dpi=0, fontsize=12,
                 scale=0):

        self.filename = filename
        self.scale = scale
        self.width = width
        self.height = height
        self.dpi = dpi
        self.fontsize = fontsize

        self.fig = plt.figure(
            figsize=(self.width, self.height), dpi=self.dpi, frameon=False
        )
        self.ax = self.fig.add_subplot(111)
        self.rr = self.fig.canvas.get_renderer()

    def save(self):

        self.fig.savefig(
            self.filename,
            dpi=self.dpi,
            bbox_inches='tight'
        )

        plt.close(self.fig)
        plt.clf()

    def _scale(self, seq_length):
        """
        scale the sequence (protein or DNA)
        """

        if self.scale is None or self.scale < seq_length:
            self.normalize = seq_length
        else:
            self.normalize = self.scale

        self.offset = 0.05 + (1.0 - float(seq_length)/self.normalize)*0.45

        assert self.normalize >= seq_length, "length normalization should be >= protein length"


class _PlotExons(_Plot):
    def __init__(self, *args, **kwargs):
        super(_PlotExons, self).__init__(*args, **kwargs)
        self.vertical_offset = 0.15

    def _draw_length_markers(self, basepair_length):
        # plot protein length markers

        self.basepair_length = basepair_length/float(self.normalize)*0.9

        self.line_end = basepair_length/float(self.normalize)*0.9 + self.offset

        self.ax.text(
            0.5,
            0.1,
            "Base pair position (kbp)",
            horizontalalignment='center',
            fontsize=self.fontsize
        )

        self.ax.add_line(plt.Line2D(
            (
                self.offset,
                self.offset+self.basepair_length
            ),
            (0.2+self.vertical_offset, 0.2+self.vertical_offset),
            color='black'
            )
        )

        # left marker

        self.left_marker_line = self.ax.add_line(plt.Line2D(
            (
                self.offset,
                self.offset
            ),
            (0.15+self.vertical_offset, 0.2+self.vertical_offset),
            color='black'
            )
        )
        self.left_marker_text = self.ax.text(
            self.offset,
            0.05+self.vertical_offset,
            "0",
            horizontalalignment='center',
            fontsize=self.fontsize
        )

        # draw markers for increments of 1000 base pairs

        for i in range(1, basepair_length+1):
            if (i % 10000) == 0:
                self.left_marker_line = self.ax.add_line(plt.Line2D(
                    (
                        self.offset+(i/float(self.normalize)*0.9),
                        self.offset+(i/float(self.normalize)*0.9)
                    ),
                    (0.175+self.vertical_offset, 0.2+self.vertical_offset),
                    color='black'
                    )
                )

        # right marker

        self.right_marker_line = self.ax.add_line(plt.Line2D(
            (
                self.offset+self.basepair_length,
                self.offset+self.basepair_length
            ),
            (0.15+self.vertical_offset, 0.2+self.vertical_offset),
            color='black'
            )
        )
        self.right_marker_text = self.ax.text(
            self.offset+self.basepair_length,
            0.05+self.vertical_offset,
            str(basepair_length/1000),
            horizontalalignment='center',
            fontsize=self.fontsize
        )


class PlotWTExons(_PlotExons):
    def __init__(self, ensembl_transcript, *args, **kwargs):
        super(PlotWTExons, self).__init__(*args, **kwargs)
        self.ensembl_transcript = ensembl_transcript

    def _draw_main_body(self, name_symbols, name_isoform):
        """
        main protein frame
        """

        length = (self.ensembl_transcript.end-self.ensembl_transcript.start)/float(self.normalize)*0.9

        self.ax.add_line(plt.Line2D(
            (
                self.offset,
                self.offset+length
            ),
            (0.5, 0.5),
            color='black'
            )
        )

        self.ax.text(
            0.5,
            0.9,
            name_symbols,
            horizontalalignment='center',
            fontsize=self.fontsize
        )
        self.ax.text(
            0.5,
            0.83,
            name_isoform,
            horizontalalignment='center',
            fontsize=self.fontsize-3
        )

    def _draw_exons(self):
        for exon in self.ensembl_transcript.exon_intervals:

            if self.ensembl_transcript.strand == '+':
                start = exon[0] - self.ensembl_transcript.start
                end = exon[1] - self.ensembl_transcript.start
            else:

                # this is so the transcription direction is not plotted
                # in reverse for genes on minus strand

                start = -(exon[1] - self.ensembl_transcript.end)
                end = -(exon[0] - self.ensembl_transcript.end)

            exon_start = (int(start)/float(self.normalize))*0.9 + self.offset
            exon_end = (int(end)/float(self.normalize))*0.9 + self.offset
            exon_center = (exon_end-exon_start)/2. + exon_start

            self.ax.add_patch(
                patches.Rectangle(
                    (
                        exon_start,
                        0.45,
                    ),
                    exon_end-exon_start,
                    0.1,
                    color="black"
                )
            )


    def draw(self):
        self._scale(self.ensembl_transcript.end-self.ensembl_transcript.start)
        self._draw_exons()
        self._draw_length_markers(self.ensembl_transcript.end-self.ensembl_transcript.start)
        self._draw_main_body(
            self.ensembl_transcript.gene.gene_name,
            self.ensembl_transcript.id
        )
        self.ax.axis('off')
        self.ax.set_xlim(0, 1)
        self.ax.set_ylim(0, 1)


class PlotFusionExons(_PlotExons):
    def __init__(self, transcript, *args, **kwargs):
        super(PlotFusionExons, self).__init__(*args, **kwargs)
        self.transcript = transcript

    def _draw_fusion_junction(self, junction_location):

        junction_location_norm = junction_location/float(self.normalize)*0.9

        self.ax.add_line(plt.Line2D(
            (
                self.offset+junction_location_norm,
                self.offset+junction_location_norm
            ),
            (0.15+self.vertical_offset, 0.2+self.vertical_offset),
            color='black'
            )
        )
        self.right_marker_text = self.ax.text(
            self.offset+junction_location_norm,
            0.05+self.vertical_offset,
            str(junction_location/1000),
            horizontalalignment='center',
            fontsize=self.fontsize
        )

    def _draw_exons(self):
        for exon in self.transcript.gene5prime_exon_intervals:

            if self.transcript.transcript1.strand == '+':
                start = exon[0] - self.transcript.transcript1.start
                end = exon[1] - self.transcript.transcript1.start
            else:

                # this is so the transcription direction is not plotted
                # in reverse for genes on minus strand

                start = -(exon[1] - self.transcript.transcript1.end)
                end = -(exon[0] - self.transcript.transcript1.end)

            exon_start = (int(start)/float(self.normalize))*0.9 + self.offset
            exon_end = (int(end)/float(self.normalize))*0.9 + self.offset
            exon_center = (exon_end-exon_start)/2. + exon_start

            self.ax.add_patch(
                patches.Rectangle(
                    (
                        exon_start,
                        0.45,
                    ),
                    exon_end-exon_start,
                    0.1,
                    color="black"
                )
            )

        if self.transcript.transcript1.strand == '+':
            distance_to_add = self.transcript.gene5prime.junction - \
                self.transcript.transcript1.start
        else:
            distance_to_add = self.transcript.transcript1.end - \
                self.transcript.gene5prime.junction

        for exon in self.transcript.gene3prime_exon_intervals:

            if self.transcript.transcript2.strand == '+':
                start = exon[0] - self.transcript.transcript2.start + \
                    distance_to_add
                end = exon[1] - self.transcript.transcript2.start + \
                    distance_to_add
            else:

                # this is so the transcription direction is not plotted
                # in reverse for genes on minus strand

                start = (self.transcript.gene3prime.junction - exon[1]) + \
                    distance_to_add
                end = (self.transcript.gene3prime.junction - exon[0]) + \
                    distance_to_add

            exon_start = (int(start)/float(self.normalize))*0.9 + self.offset
            exon_end = (int(end)/float(self.normalize))*0.9 + self.offset
            exon_center = (exon_end-exon_start)/2. + exon_start

            self.ax.add_patch(
                patches.Rectangle(
                    (
                        exon_start,
                        0.45,
                    ),
                    exon_end-exon_start,
                    0.1,
                    color="red"
                )
            )

    def _draw_main_body(self, name_symbols, name_isoform, length):
        """
        main protein frame
        """

        gene5prime_length = 0
        gene3prime_length = 0

        if self.transcript.transcript1.strand == '+':
            gene5prime_length = (self.transcript.gene5prime.junction -
                                 self.transcript.transcript1.start) \
                                 / float(self.normalize)*0.9
        else:
            gene5prime_length = (self.transcript.transcript1.end -
                                 self.transcript.gene5prime.junction) \
                                 / float(self.normalize)*0.9

        if self.transcript.transcript2.strand == '+':
            gene3prime_length = (self.transcript.transcript2.end -
                                 self.transcript.gene3prime.junction) \
                                 / float(self.normalize)*0.9
        else:
            gene3prime_length = (self.transcript.gene3prime.junction -
                                 self.transcript.transcript2.start) \
                                 / float(self.normalize)*0.9

        self.ax.add_line(plt.Line2D(
            (
                self.offset,
                self.offset+gene5prime_length
            ),
            (0.5, 0.5),
            color='black'
            )
        )
        self.ax.add_line(plt.Line2D(
            (
                self.offset+gene5prime_length,
                self.offset+gene5prime_length+gene3prime_length
            ),
            (0.5, 0.5),
            color='red'
            )
        )

        self.ax.text(
            0.5,
            0.9,
            name_symbols,
            horizontalalignment='center',
            fontsize=self.fontsize
        )
        self.ax.text(
            0.5,
            0.83,
            name_isoform,
            horizontalalignment='center',
            fontsize=self.fontsize-3
        )

    def draw(self):

        if self.transcript.transcript1.strand == '+':
            gene5prime_length = self.transcript.gene5prime.junction - \
                self.transcript.transcript1.start
        else:
            gene5prime_length = self.transcript.transcript1.end - \
                self.transcript.gene5prime.junction

        if self.transcript.transcript2.strand == '+':
            gene3prime_length = self.transcript.transcript2.end - \
                self.transcript.gene3prime.junction
        else:
            gene3prime_length = self.transcript.gene3prime.junction - \
                self.transcript.transcript2.start

        self._scale(gene5prime_length+gene3prime_length)
        self._draw_exons()
        self._draw_length_markers(gene5prime_length+gene3prime_length)
        self._draw_fusion_junction(gene5prime_length)
        self._draw_main_body(
            self.transcript.transcript1.gene.gene_name + '-' +
            self.transcript.transcript2.gene.gene_name,
            self.transcript.transcript1.id + '-' +
            self.transcript.transcript2.id,
            gene5prime_length+gene3prime_length
        )
        self.ax.axis('off')
        self.ax.set_xlim(0, 1)
        self.ax.set_ylim(0, 1)


class _PlotProtein(_Plot):

    def __init__(self, transcript=None, colors=None,
                 rename=None, no_domain_labels=False,
                 exclude = [], *args, **kwargs):
        super(_PlotProtein, self).__init__(*args, **kwargs)
        self.transcript = transcript
        self.colors = colors
        self.rename = rename
        self.no_domain_labels = no_domain_labels
        self.exclude = exclude
        self.vertical_offset = 0.55

    def _draw_domains(self, domains):
        # plot domains

        domain_labels = {i:[] for i in HORIZONTAL_LEVELS}
        domain_labels_levels = {}
        domain_label_boxes = {i:[] for i in HORIZONTAL_LEVELS}
        lowest_level_plotted = HORIZONTAL_LEVELS[0]

        domains.sort(key=lambda x: x[3])

        domain_count = 0

        for domain in domains:

            # use domain name if available, otherwise use its ID

            if domain[1] is None:
                domain_name = str(domain[0])
            else:
                domain_name = str(domain[1])

            if domain_name in self.exclude:
                continue

            if domain_name in self.rename:
                domain_name = self.rename[domain_name]

            domain_start = (int(domain[3])/float(self.normalize))*0.9 + self.offset
            domain_end = (int(domain[4])/float(self.normalize))*0.9 + self.offset
            domain_center = (domain_end-domain_start)/2. + domain_start

            if not self.no_domain_labels:

                # for each newly plotted domain, loop through all previous
                # plotted domains and calculated the extent of overlap
                # then horizontally adjust the domain label to be plotted
                # closest to the protein domain structure or be
                # on the same level as the label it overlaps with the least

                #domain_stack_level = cycle()
                overlaps = {i:0.0 for i in HORIZONTAL_LEVELS}
                overlaps_all_levels = True
                min_overlap = [float("inf"),HORIZONTAL_LEVELS[0]]
                # plot domain at 1st level

                for level in HORIZONTAL_LEVELS:

                    level_pos = self.vertical_offset - 0.15 - (level-1.0)*0.1

                    tmp_domain_label = self.ax.text(
                        domain_center,
                        level_pos,
                        domain_name,
                        horizontalalignment='center',
                        verticalalignment='center',
                        fontsize=self.fontsize
                    )
                    tmp_domain_label_box = tmp_domain_label.get_window_extent(renderer=self.rr)

                    #check to see if it overlaps with anything

                    if len(domain_label_boxes[level])>0:
                        max_overlap = max([i.x1 - tmp_domain_label_box.x0 for i in domain_label_boxes[level]])

                        if max_overlap > 0.0:
                            overlaps[level] = max_overlap
                            tmp_domain_label.remove()

                            if max_overlap <= min_overlap[0]:
                                min_overlap = [max_overlap, level]
                        else:
                            domain_labels[level].append(tmp_domain_label)
                            domain_label_boxes[level].append(tmp_domain_label_box)
                            overlaps_all_levels = False

                            domain_labels_levels[domain_count] = level

                            if level > lowest_level_plotted:
                                lowest_level_plotted = level

                            break
                    else:
                        domain_labels[level].append(tmp_domain_label)
                        domain_label_boxes[level].append(tmp_domain_label_box)
                        overlaps_all_levels = False

                        domain_labels_levels[domain_count] = level

                        if level > lowest_level_plotted:
                            lowest_level_plotted = level

                        break

                # if the domain label overlaps with something on all levels
                # then plot it on the level where is overlaps the least

                if overlaps_all_levels:

                    level_pos = self.vertical_offset - 0.15 - (min_overlap[1]-1.0)*0.1

                    tmp_domain_label = self.ax.text(
                        domain_center,
                        level_pos,
                        domain_name,
                        horizontalalignment='center',
                        verticalalignment='center',
                        fontsize=self.fontsize
                    )
                    tmp_domain_label_box = tmp_domain_label.get_window_extent(renderer=self.rr)

                    domain_labels[min_overlap[1]].append(tmp_domain_label)
                    domain_label_boxes[min_overlap[1]].append(tmp_domain_label_box)

                    domain_labels_levels[domain_count] = level

                    if min_overlap[1] > lowest_level_plotted:
                        lowest_level_plotted = min_overlap[1]
            domain_count += 1

        # now we know how many levels of domains labels are needed, so
        # remove all levels, make the correction to self.vertical_offset
        # and replot all labels.

        for level, label in list(domain_labels.items()):
            for ll in label:
                ll.remove()

        self.levels_plotted = HORIZONTAL_LEVELS.index(lowest_level_plotted)
        self.vertical_offset += (0.05 * self.levels_plotted)

        domain_count = 0

        for domain in domains:

            if domain[1] is None:
                domain_name = str(domain[0])
            else:
                domain_name = str(domain[1])

            if domain_name in self.exclude:
                continue

            color = '#3385ff'
            if domain_name in self.colors:
                color = self.colors[domain_name]

            domain_start = (int(domain[3])/float(self.normalize))*0.9 + self.offset
            domain_end = (int(domain[4])/float(self.normalize))*0.9 + self.offset
            domain_center = (domain_end-domain_start)/2. + domain_start

            self.ax.add_patch(
                patches.Rectangle(
                    (
                        domain_start,
                        self.vertical_offset,
                    ),
                    domain_end - domain_start,
                    0.1,
                    color=color
                )
            )

            # fetch the level the domain label was determined it was to be
            # plotted on

            level = domain_labels_levels[domain_count]

            level_pos = self.vertical_offset - 0.15 - (level-1.0)*0.1

            tmp_domain_label = self.ax.text(
                domain_center,
                level_pos,
                domain_name,
                horizontalalignment='center',
                verticalalignment='center',
                fontsize=self.fontsize
            )

            domain_count += 1


    def _draw_protein_length_markers(self, protein_length):
        # plot protein length markers

        self.line_end = protein_length/float(self.normalize)*0.9 + self.offset

        self.ax.text(
            0.5,
            self.vertical_offset - (0.5 + self.levels_plotted * 0.1),
            "Amino acid position",
            horizontalalignment='center',
            verticalalignment='center',
            fontsize=self.fontsize
        )

        self.ax.add_line(plt.Line2D(
            (
                self.offset,
                self.offset+self.protein_frame_length
            ),
            (
                self.vertical_offset - (0.35 + self.levels_plotted * 0.05),
                self.vertical_offset - (0.35 + self.levels_plotted * 0.05)
            ),
            color='black'
            )
        )

        # left marker

        self.left_marker_line = self.ax.add_line(plt.Line2D(
            (
                self.offset,
                self.offset
            ),
            (
                self.vertical_offset - (0.38 + self.levels_plotted * 0.05),
                self.vertical_offset - (0.35 + self.levels_plotted * 0.05)
            ),
            color='black'
            )
        )
        self.left_marker_text = self.ax.text(
            self.offset,
            self.vertical_offset - (0.43 + self.levels_plotted * 0.05),
            "0",
            horizontalalignment='center',
            verticalalignment='center',
            fontsize=self.fontsize
        )

        # draw markers for increments of 100 amino acids

        for i in range(1, protein_length+1):
            if (i % 100) == 0:
                self.left_marker_line = self.ax.add_line(plt.Line2D(
                    (
                        self.offset+(i/float(self.normalize)*0.9),
                        self.offset+(i/float(self.normalize)*0.9)
                    ),
                    (
                        self.vertical_offset - (0.38 + self.levels_plotted * 0.05),
                        self.vertical_offset - (0.35 + self.levels_plotted * 0.05)
                    ),
                    color='black'
                    )
                )

        # right marker

        self.right_marker_line = self.ax.add_line(plt.Line2D(
            (
                self.offset+self.protein_frame_length,
                self.offset+self.protein_frame_length
            ),
            (
                self.vertical_offset - (0.38 + self.levels_plotted * 0.05),
                self.vertical_offset - (0.35 + self.levels_plotted * 0.05)
            ),
            color='black'
            )
        )

        self.right_marker_text = self.ax.text(
            self.offset+self.protein_frame_length,
            self.vertical_offset - (0.43 + self.levels_plotted * 0.05),
            str(protein_length),
            horizontalalignment='center',
            verticalalignment='center',
            fontsize=self.fontsize
        )

    def _draw_main_body(self, name_symbols, name_isoform):
        """
        main protein frame
        """

        self.ax.add_patch(
            patches.Rectangle(
                (self.offset, self.vertical_offset),
                self.protein_frame_length,
                0.1,
                fill=False
            )
        )

        self.ax.text(
            0.5,
            0.95,
            name_symbols,
            horizontalalignment='center',
            fontsize=self.fontsize
        )

        self.ax.text(
            0.5,
            0.88,
            name_isoform,
            horizontalalignment='center',
            fontsize=self.fontsize-3
        )


class PlotFusionProtein(_PlotProtein):
    def __init__(self, *args, **kwargs):
        super(PlotFusionProtein, self).__init__(*args, **kwargs)

    def _draw_junction(self):
        # add the junction

        self.ax.add_line(plt.Line2D(
            (
                (self.transcript.transcript_protein_junction_5prime/float(self.normalize))*0.9 + self.offset,
                (self.transcript.transcript_protein_junction_5prime/float(self.normalize))*0.9 + self.offset
            ),
            (
                self.vertical_offset - 0.05,
                self.vertical_offset + 0.15
            ),
            color='black'
            )
        )

        # middle marker, loop until it does not overlap with right marker

        overlaps = True
        line_offset = (self.transcript.transcript_protein_junction_5prime/float(self.normalize))*0.9 + self.offset
        text_offset = (self.transcript.transcript_protein_junction_5prime/float(self.normalize))*0.9 + self.offset
        junction_label_vertical_offset = 0.0

        right_marker_text_box = self.right_marker_text.get_window_extent(renderer=self.rr)
        left_marker_text_box = self.left_marker_text.get_window_extent(renderer=self.rr)

        while overlaps:

            # middle_marker_line_1/2/3 are to draw angled line

            middle_marker_line_1 = self.ax.add_line(plt.Line2D(
                (
                    (self.transcript.transcript_protein_junction_5prime/float(self.normalize))*0.9 + self.offset,
                    (self.transcript.transcript_protein_junction_5prime/float(self.normalize))*0.9 + self.offset
                ),
                (
                    self.vertical_offset - (0.37 + self.levels_plotted * 0.05) - junction_label_vertical_offset,
                    self.vertical_offset - (0.35 + self.levels_plotted * 0.05)
                ),
                color='black'
                )
            )

            middle_marker_line_2 = self.ax.add_line(plt.Line2D(
                (
                    line_offset,
                    (self.transcript.transcript_protein_junction_5prime/float(self.normalize))*0.9 + self.offset
                ),
                (
                    self.vertical_offset - (0.37 + self.levels_plotted * 0.05) - junction_label_vertical_offset,
                    self.vertical_offset - (0.37 + self.levels_plotted * 0.05) - junction_label_vertical_offset
                ),
                color='black'
                )
            )

            middle_marker_line_3 = self.ax.add_line(plt.Line2D(
                (
                    line_offset,
                    line_offset
                ),
                (
                    self.vertical_offset - (0.4 + self.levels_plotted * 0.05) - junction_label_vertical_offset,
                    self.vertical_offset - (0.37 + self.levels_plotted * 0.05) - junction_label_vertical_offset
                ),
                color='black'
                )
            )

            middle_marker_text = self.ax.text(
                text_offset,
                self.vertical_offset - (0.45 + self.levels_plotted * 0.05) - junction_label_vertical_offset,
                str(self.transcript.transcript_protein_junction_5prime),
                horizontalalignment='center',
                verticalalignment='center',
                fontsize=self.fontsize
            )

            # detect if text overlaps

            middle_marker_text_box = middle_marker_text.get_window_extent(
                renderer=self.rr
            )

            # if overlaps then offset the junction text to the left

            if (right_marker_text_box.fully_overlaps(middle_marker_text_box)) and (left_marker_text_box.fully_overlaps(middle_marker_text_box)):
                junction_label_vertical_offset = junction_label_vertical_offset + 0.01

                middle_marker_line_1.remove()
                middle_marker_line_2.remove()
                middle_marker_line_3.remove()
                middle_marker_text.remove()

            elif right_marker_text_box.fully_overlaps(middle_marker_text_box):
                line_offset = line_offset - 0.01
                text_offset = text_offset - 0.01

                middle_marker_line_1.remove()
                middle_marker_line_2.remove()
                middle_marker_line_3.remove()
                middle_marker_text.remove()
            elif left_marker_text_box.fully_overlaps(middle_marker_text_box):
                line_offset = line_offset + 0.01
                text_offset = text_offset + 0.01

                middle_marker_line_1.remove()
                middle_marker_line_2.remove()
                middle_marker_line_3.remove()
                middle_marker_text.remove()
            else:
                overlaps = False

    def draw(self):
        self._scale(self.transcript.protein_length)
        self.protein_frame_length = self.transcript.protein_length/float(self.normalize)*0.9
        self._draw_domains(self.transcript.domains['fusion'])
        self._draw_protein_length_markers(self.transcript.protein_length)
        self._draw_junction()

        name_symbols = self.transcript.gene5prime.gene.gene_name + ' - ' + \
            self.transcript.gene3prime.gene.gene_name
        name_isoform = self.transcript.transcript1.id + ' - ' + \
            self.transcript.transcript2.id
        self._draw_main_body(name_symbols, name_isoform)

        self.ax.axis('off')
        self.ax.set_xlim(0, 1)
        self.ax.set_ylim(0, 1)


class PlotWTProtein(_PlotProtein):
    def __init__(self, ensembl_transcript, *args, **kwargs):
        super(PlotWTProtein, self).__init__(*args, **kwargs)
        self.ensembl_transcript = ensembl_transcript

    def draw(self):
        self._scale(len(self.ensembl_transcript.coding_sequence)/3)
        self.protein_frame_length = len(self.ensembl_transcript.coding_sequence)/3/float(self.normalize)*0.9
        self._draw_domains(self.transcript.domains[self.ensembl_transcript.id])
        self._draw_protein_length_markers(len(self.ensembl_transcript.coding_sequence)/3)
        self._draw_main_body(
            self.ensembl_transcript.gene.gene_name,
            self.ensembl_transcript.id
        )

        self.ax.axis('off')
        self.ax.set_xlim(0, 1)
        self.ax.set_ylim(0, 1)



import sys
import gzip
from os.path import abspath, exists, join, split
import sqlite3
import logging

from agfusion.utils import PROTEIN_ANNOTATIONS, ENSEMBL_MYSQL_TABLES

class AGFusionDB():
    """
    Class to handle methods around interacting with the AGFusion SQLite3
    database

    name: name of the database
    reference_name
    """

    def __init__(self, database=None, debug=False):

        self.database = abspath(database)
        self.fastas = {}

        self.logger = logging.getLogger('AGFusion')
        if debug:
            self.logger.setLevel(logging.DEBUG)
        else:
            self.logger.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        if debug:
            ch.setLevel(logging.DEBUG)
        else:
            ch.setLevel(logging.INFO)
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        )
        ch.setFormatter(formatter)
        self.logger.addHandler(ch)

        assert exists(self.database), "AGFusion database at %s does not exist! Either run \'agfusion download\' or specify the location of the AGFusion database with the --dbpath flag." % database

        self.sqlite3_db = sqlite3.connect(
            abspath(self.database)
        )
        self.sqlite3_cursor = self.sqlite3_db.cursor()
        self.sqlite3_db.commit()

        self.logger.debug(
            'Connected to the database ' + abspath(self.database)
        )

        self.build = ''


class AGFusionDBBManager():
    """
    Class to handle methods managing the SQLite3 database

    name: name of the database
    reference
    """

    def __init__(self, db_dir, species, release, pfam, server):

        self.species = species
        self.release = release
        self.server = server
        self.build = self.species + '_' + str(self.release)
        self.table = ENSEMBL_MYSQL_TABLES[self.species][self.release]

        self.database = join(
            abspath(db_dir),
            'agfusion.' + self.species + '.' + str(self.release) + '.db'
        )
        self.fastas = {}

        self.logger = logging.getLogger('AGFusion')
        self.logger.setLevel(logging.INFO)
        ch = logging.StreamHandler()
        ch.setLevel(logging.INFO)
        formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        )
        ch.setFormatter(formatter)
        self.logger.addHandler(ch)

        if not exists(self.database):
            fout = open(abspath(self.database), 'a')
            fout.close()
            self.logger.info(
                "Created the database " + abspath(self.database)
            )

        self.sqlite3_db = sqlite3.connect(
            abspath(self.database)
        )
        self.sqlite3_cursor = self.sqlite3_db.cursor()
        self.sqlite3_db.commit()

        self.logger.info(
            'Connected to the database ' + abspath(self.database)
        )

        import MySQLdb

        self.ensembl_db = MySQLdb.connect(server, 'anonymous')
        self.ensembl_cursor = self.ensembl_db.cursor()
        self.ensembl_cursor.execute('use ' + self.table + ';')

        self.logger.info(
            'Connected to the ensembl MySQL server at ' + server
        )
        self.logger.info('MySQL - use ' +  self.table + ';')

        self._check_for_tables()

        #load the PFAM mappings

        self.pfam_mapping = dict()

        for line in open(pfam, 'r'):
            line = line.rstrip().split('\t')

            pfam_id = line[0]
            pfam_name = line[1]
            pfam_desc = line[3]

            self.pfam_mapping[pfam_id] = {
                'name': pfam_name,
                'desc': pfam_desc
            }

    def _check_for_tables(self):

        self.sqlite3_cursor.execute('drop table if exists ' + self.build)

        sqlite3_command = "CREATE TABLE " + self.build + " (" + \
            "gene_id text," + \
            "stable_id text," + \
            "entrez_id text," + \
            "gene_name text," + \
            "canonical_transcript_id text);"

        self.logger.info('SQLite - ' + sqlite3_command)

        self.sqlite3_cursor.execute(
            sqlite3_command
        )
        self.sqlite3_db.commit()

        # transcript table

        self.sqlite3_cursor.execute(
            'drop table if exists ' +
            self.build + '_transcript'
        )

        sqlite3_command = "CREATE TABLE " + self.build + "_transcript (" + \
            "transcript_id text," + \
            "gene_id text," + \
            "transcript_stable_id text," + \
            "translation_id text);"

        self.logger.info('SQLite - ' + sqlite3_command)

        self.sqlite3_cursor.execute(
            sqlite3_command
        )
        self.sqlite3_db.commit()

        # refseq table

        self.sqlite3_cursor.execute(
            'drop table if exists ' +
            self.build + '_refseq'
        )

        sqlite3_command = "CREATE TABLE " + self.build + "_refseq (" + \
            "transcript_id text," + \
            "transcript_stable_id text," + \
            "refseq_id text);"

        self.logger.info('SQLite - ' + sqlite3_command)

        self.sqlite3_cursor.execute(
            sqlite3_command
        )
        self.sqlite3_db.commit()

        # protein annotation tables

        for protein_annotation in PROTEIN_ANNOTATIONS:
            self.sqlite3_cursor.execute(
                'drop table if exists ' +
                self.build + '_' + protein_annotation
            )

            sqlite3_command = "CREATE TABLE " + self.build + "_" + \
                protein_annotation + " (" + \
                "translation_id text," + \
                "stable_id text," + \
                "hit_id text," + \
                "seq_start integer," + \
                "seq_end integer," + \
                "hit_description text," + \
                "hit_name text);"

            self.logger.info('SQLite - ' + sqlite3_command)

            self.sqlite3_cursor.execute(
                sqlite3_command
            )
            self.sqlite3_db.commit()


    def fetch_gene_names(self):

        genes = {}

        # fetch all gene stable ids

        mysql_command = "SELECT gene.gene_id, gene.stable_id, gene.canonical_transcript_id FROM gene;"

        self.logger.info('MySQL - ' + mysql_command)

        self.ensembl_cursor.execute(
            mysql_command
        )
        for g in self.ensembl_cursor.fetchall():
            genes[g[0]] = {
                'stable_id': g[1],
                'canonical_transcript_id': g[2],
                'entrez_id': '',
                'gene_name': ''
            }

        # fetch entrez IDS

        mysql_command = "SELECT gene.gene_id, xref.dbprimary_acc FROM gene, object_xref, xref,external_db WHERE gene.gene_id = object_xref.ensembl_id AND object_xref.ensembl_object_type = 'Gene' AND object_xref.xref_id = xref.xref_id AND xref.external_db_id = external_db.external_db_id AND external_db.db_name = 'EntrezGene';"
        self.logger.info('MySQL - ' + mysql_command)

        self.ensembl_cursor.execute(mysql_command)

        for g in self.ensembl_cursor.fetchall():
            genes[g[0]]['entrez_id'] = g[1]

        # fetch gene names

        if self.build.find('homo_sapiens') != -1:
            gene_name_db = 'HGNC'
        elif self.build.find('mus_musculus') != -1:
            gene_name_db = 'MGI'
        else:
            self.logger.error(
                'Do not know where to fetch gene names for ' + self.build
            )
            sys.exit()

        mysql_command = """SELECT gene.gene_id, xref.display_label FROM gene, object_xref, xref,external_db WHERE gene.gene_id = object_xref.ensembl_id AND object_xref.ensembl_object_type = 'Gene' AND object_xref.xref_id = xref.xref_id AND xref.external_db_id = external_db.external_db_id AND external_db.db_name = '""" + gene_name_db + """';"""

        self.logger.info('MySQL - ' + mysql_command)

        self.ensembl_cursor.execute(mysql_command)

        for g, e in self.ensembl_cursor.fetchall():
            genes[g]['gene_name'] = e

        genes = [
            [
                i,
                genes[i]['stable_id'],
                genes[i]['entrez_id'],
                genes[i]['gene_name'],
                genes[i]['canonical_transcript_id']
            ] for i in list(genes.keys())
        ]

        self.logger.info(
            'SQLite - INSERT INTO ' +
            self.build + ' VALUES (gene_id,stable_id,entrez_id,gene_name,canonical_transcript_id)'
        )

        self.sqlite3_cursor.executemany(
            'INSERT INTO ' + self.build + ' VALUES (?,?,?,?,?)', genes
        )

        self.sqlite3_db.commit()

    def fetch_transcript_table(self):

        # Fetch all transcripts

        mysql_command = "SELECT transcript.transcript_id, transcript.gene_id, transcript.stable_id FROM transcript;"
        self.logger.info('MySQL - ' + mysql_command)
        self.ensembl_cursor.execute(
            mysql_command
        )
        transcripts = {}
        for transcript in self.ensembl_cursor.fetchall():
            transcripts[transcript[0]] = {
                'gene_id': transcript[1],
                'transcript_stable_id': transcript[2],
                'translation_id': ''
            }

        # Fetch all transcripts with tranlstions

        mysql_command = "SELECT transcript.transcript_id, translation.translation_id FROM transcript, translation WHERE transcript.transcript_id = translation.transcript_id;"
        self.logger.info('MySQL - ' + mysql_command)
        self.ensembl_cursor.execute(
            mysql_command
        )
        for transcript in self.ensembl_cursor.fetchall():
            transcripts[transcript[0]]['translation_id'] = transcript[1]

        transcripts = [
            [
                i,
                transcripts[i]['gene_id'],
                transcripts[i]['transcript_stable_id'],
                transcripts[i]['translation_id']
            ] for i in list(transcripts.keys())
        ]

        self.logger.info(
            'SQLite - INSERT INTO ' +
            self.build + '_transcript VALUES (transcript_id,gene_id,transcript_stable_id,translation_id)'
        )

        self.sqlite3_cursor.executemany(
            'INSERT INTO ' + self.build + '_transcript VALUES (?,?,?,?)', transcripts
        )

        self.sqlite3_db.commit()

    def fetch_refseq_table(self):

        # fetch RefSeq IDS

        mysql_command = "SELECT transcript.transcript_id, transcript.stable_id, xref.display_label FROM transcript, object_xref, xref, external_db WHERE transcript.transcript_id = object_xref.ensembl_id AND object_xref.ensembl_object_type = \'Transcript\' AND object_xref.xref_id = xref.xref_id AND xref.external_db_id = external_db.external_db_id AND external_db.db_name = \'RefSeq_mRNA\';"
        self.logger.info('MySQL - ' + mysql_command)
        self.ensembl_cursor.execute(
            mysql_command
        )

        refseqs = [
            [
                i[0],
                i[1],
                i[2]
            ] for i in self.ensembl_cursor.fetchall()
        ]

        self.logger.info(
            'SQLite - INSERT INTO ' +
            self.build + '_refseq VALUES (transcript_id,transcript_stable_id,refseq_id)'
        )

        self.sqlite3_cursor.executemany(
            'INSERT INTO ' + self.build + '_refseq VALUES (?,?,?)', refseqs
        )

        self.sqlite3_db.commit()

    def fetch_protein_annotation(self):

        for protein_annotation in PROTEIN_ANNOTATIONS:
            mysql_command = "SELECT translation.translation_id, translation.stable_id, protein_feature.hit_name, protein_feature.seq_start, protein_feature.seq_end, protein_feature.hit_description FROM analysis, analysis_description, protein_feature, translation WHERE protein_feature.translation_id = translation.translation_id AND protein_feature.analysis_id = analysis.analysis_id AND analysis.analysis_id = analysis_description.analysis_id AND analysis.logic_name = \'" + protein_annotation + "\';"

            self.logger.info('MySQL - ' + mysql_command)

            self.ensembl_cursor.execute(
                mysql_command
            )

            data = list(self.ensembl_cursor.fetchall())

            if protein_annotation=='pfam':
                for i in range(0,len(data)):
                    tmp = list(data[i])
                    if tmp[2] in self.pfam_mapping:
                        tmp.append(self.pfam_mapping[tmp[2]]['name'])
                        tmp[5] = self.pfam_mapping[tmp[2]]['desc']
                    else:
                        tmp.append('')
                    data[i] = tmp
            else:
                for i in range(0,len(data)):
                    tmp = list(data[i])
                    tmp.append(None)
                    data[i] = tmp

            self.logger.info(
                'SQLite - INSERT INTO ' +
                self.build + '_' + protein_annotation + ' VALUES (translation_id,stable_id,hit_id,seq_start,seq_end,hit_description,hit_name)'
            )

            self.sqlite3_cursor.executemany(
                'INSERT INTO ' + self.build +
                '_' + protein_annotation + ' VALUES (?,?,?,?,?,?,?)', data
            )

            self.sqlite3_db.commit()



"""
Parses output files from fusion-finding algorithms
"""

import re

class _Parser(object):
    def __init__(self):
        self.fusions = []
        self.iterator = 0

    def __iter__(self):
        return self

    def __next__(self):
        if self.iterator >= len(self.fusions):
            raise StopIteration
        else:
            self.iterator += 1
            return self.fusions[self.iterator-1]

    next = __next__

class STARFusion(_Parser):
    def __init__(self,infile):
        super(STARFusion, self).__init__()

        fin = open(infile,'r')
        for line in fin.readlines():
            if re.findall('^#',line):
                line = line.rstrip().split('\t')
                assert line[0]=='#FusionName', 'Unrecognized STAR-Fusion input'
                assert line[4]=='LeftGene', 'Unrecognized STAR-Fusion input'
                assert line[5]=='LeftBreakpoint', 'Unrecognized STAR-Fusion input'
                assert line[6]=='RightGene', 'Unrecognized STAR-Fusion input'
                assert line[7]=='RightBreakpoint', 'Unrecognized STAR-Fusion input'
                continue

            line = line.strip().split('\t')

            gene_5prime = line[4].split('^')[1].split('.')[0]
            gene_5prime_name = line[4].split('^')[0]
            gene_5prime_junction = int(line[5].split(':')[1])
            gene_3prime = line[6].split('^')[1].split('.')[0]
            gene_3prime_name = line[6].split('^')[0]
            gene_3prime_junction = int(line[7].split(':')[1])
            self.fusions.append(
                {
                    'ensembl_5prime':gene_5prime,
                    'ensembl_3prime':gene_3prime,
                    'alternative_name_5prime':gene_5prime_name,
                    'alternative_name_3prime':gene_3prime_name,
                    'junction_5prime':gene_5prime_junction,
                    'junction_3prime':gene_3prime_junction
                }
            )
        fin.close()

class FusionEntry():
    pass

class Bellerophontes(_Parser):
    pass

class Chimerascan(_Parser):
    pass

class EricScript(_Parser):
    pass

class FusionCatcher(_Parser):
    def __init__(self,infile):
        super(FusionCatcher, self).__init__()

        fin = open(infile,'r')
        for line in fin.readlines():
            if re.findall('^Gene_1_symbol',line):
                line = line.rstrip().split('\t')
                assert line[8]=='Fusion_point_for_gene_1(5end_fusion_partner)', 'Unrecognized FusionCatcher input'
                assert line[9]=='Fusion_point_for_gene_2(3end_fusion_partner)', 'Unrecognized FusionCatcher input'
                assert line[10]=='Gene_1_id(5end_fusion_partner)', 'Unrecognized FusionCatcher input'
                assert line[11]=='Gene_2_id(3end_fusion_partner)', 'Unrecognized FusionCatcher input'
                continue

            line = line.strip().split('\t')
            self.fusions.append(
                {
                    'ensembl_5prime':line[10],
                    'ensembl_3prime':line[11],
                    'alternative_name_5prime':line[0],
                    'alternative_name_3prime':line[1],
                    'junction_5prime':int(line[8].split(':')[1]),
                    'junction_3prime':int(line[9].split(':')[1])
                }
            )
        fin.close()

class FusionHunter(_Parser):
    pass

class FusionMap(_Parser):
    pass

class JAFFA(_Parser):
    pass

class MapSplice(_Parser):
    pass

class nFuse(_Parser):
    pass

class SOAPfuse(_Parser):
    pass

class TopHatFusion(_Parser):
    def __init__(self,infile):
        super(TopHatFusion, self).__init__()

        fin = open(infile,'r')
        for line in fin.readlines():
            line = line.strip().split('\t')

            gene_5prime = line[1]
            gene_5prime_name = line[1]
            gene_5prime_junction = int(line[3])
            gene_3prime = line[4]
            gene_3prime_name = line[4]
            gene_3prime_junction = int(line[6])
            self.fusions.append(
                {
                    'ensembl_5prime':gene_5prime,
                    'ensembl_3prime':gene_3prime,
                    'alternative_name_5prime':gene_5prime_name,
                    'alternative_name_3prime':gene_3prime_name,
                    'junction_5prime':gene_5prime_junction,
                    'junction_3prime':gene_3prime_junction
                }
            )
        fin.close()

parsers = {
    #'bellerophontes':Bellerophontes,
    #'chimerascan':Chimerascan,
    #'ericscript':EricScript,
    'fusioncatcher':FusionCatcher,
    #'fusionhunter':FusionHunter,
    #'fusionmap':FusionMap,
    #'jaffa':JAFFA,
    #'mapsplice':MapSplice,
    #'nfuse':nFuse,
    #'soapfuse':SOAPfuse,
    'tophatfusion':TopHatFusion,
    'starfusion':STARFusion
}
